#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 shadow_position;
} vert;

out vec4 color;

uniform vec3 camera_world;
uniform vec3 camera_front;

uniform vec4 materialCoefficients; // x = ambient, y = diffuse, z = specular, w = specular alpha

uniform sampler2D diffuseTexture;
uniform sampler2D shadowmap;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct Flashlight {
    float arc;
    float outer_arc;
    bool on;
    vec3 color;
} flashlight;

uniform float brightness;

vec3  cel(vec3 col, vec3 n, vec3 l) {
float intensity = dot(n,l);
   vec3 discrete_color;

	 if (intensity > 0.9) {
	 discrete_color = col;
	 } else if (intensity >0.8) {
	 discrete_color = col*0.7;
	 } else if (intensity >0.7) {
	  discrete_color = col*0.6;
	 } else if (intensity >0.6) {
	 discrete_color = col *0.5;
	} else if (intensity >0.5) {
	 discrete_color = col *0.4;
	}else if (intensity >0.4) {
	 discrete_color = col *0.3;
	}else if (intensity >0.3) {
	 discrete_color = col *0.2;
	}else {
	 discrete_color = col *0.1;

	}
	return discrete_color;
	}

	float shadowCalc(vec4 fragPos)
{
    // perform perspective divide
    vec3 projCoords = fragPos.xyz / fragPos.w;
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    double closestDepth = texture(shadowmap, projCoords.xy).r; 
    // get depth of current fragment from light's perspective
    double currentDepth = projCoords.z;
    // check whether current frag pos is in shadow
	vec3 normal = normalize(vert.normal_world);
    vec3 lightDir = - normalize(dirL.direction);
    float bias = max(0.002 * (1.0 - dot(normal, lightDir)), 0.0015);

	float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowmap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowmap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;
    return shadow;
}

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	vec3 ld = normalize(-dirL.direction);

	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	color = vec4(texColor , 1); 

	float shadow =  1.0f - shadowCalc(vert.shadow_position);

	// add directional light contribution
	//color.rgb = phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, materialCoefficients.w, false, vec3(0));
	
	color.rgb = shadow * cel(color.rgb*dirL.color, n, ld);
	color.rgb *= vec3(brightness, brightness, brightness);

	if(flashlight.on){
        float theta = dot(v, -camera_front);
        float epsilon = flashlight.arc - flashlight.outer_arc;
        float intensity = clamp(((theta- flashlight.outer_arc) / epsilon), 0.0f, 1.0f);
        color.rgb += intensity * phong(n, camera_world - vert.position_world, v, flashlight.color * texColor,
            materialCoefficients.y, flashlight.color, materialCoefficients.z,
            materialCoefficients.w, true, vec3(1.0f, 0.04f, 0.001f));
    }

	// add point light contribution
	//color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, materialCoefficients.w, true, pointL.attenuation);


}
